<!doctype html>
<html lang="en">
    <head>
        <title>
            {% block maintitle %} {% endblock %} PYYACP Table Profile
        </title>
        <meta charset="utf-8">
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
        <meta name="viewport" content="width=device-width, initial-scale=1">

         <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/semantic-ui@2.3.1/dist/semantic.min.css">

        {% block headcss %} {% endblock %}
        <script src="https://ajax.googleapis.com/ajax/libs/jquery/2.1.1/jquery.min.js"></script>
        <script src="https://cdn.jsdelivr.net/npm/semantic-ui@2.3.1/dist/semantic.min.js"></script>

        {% block headscripts %}  {% endblock %}

        <style>
        <style>
            .ui.card .meta {
                font-size: 0.8em;
            }

            .ellipsis {
                overflow: hidden;
                text-overflow: ellipsis;
                white-space: nowrap;
            }
            table.mini td {
                white-space: nowrap;
                padding: 0 4px 0 2px;
                border-left:1px solid #e0e0e0;
            }
            table.mini {font-size: 0.8em;}

            table.mini th {
                padding: 0 2px 0 2px;
	            border-top:1px solid #fafafa;
	            border-bottom:2px solid #e0e0e0;
	            border-left:1px solid #e0e0e0;

	            background: #ededed;
	            background: -webkit-gradient(linear, left top, left bottom, from(#ededed), to(#ebebeb));
	            background: -moz-linear-gradient(top,  #ededed,  #ebebeb);
            }


            {% block headerstyle %}

            {% endblock %}
        </style>
        <script>
            {% block headscript %}

            {% endblock %}
        </script>
    </head>

    <body>

    <!-- HEADER -->
        <header class="ui basic vertical segment" >
            <div class="ui container">
            <h1 class="ui header fifteen wide column" style="color:#1985A1;">
                <div class="content" >
                    pyYACP
                    <div class="sub header" style="color:#1985A1;">Yet another Python CSV Parser|Profiler</div>
                </div>
            </h1>
            </div>
        </header>


    <!-- MAIN NAVIGATION -->
        <nav id="mainnav" class="ui  fluid secondary stackable pointing icon menu " >

        </nav>


    <!-- CONTENT -->
         <section class="ui basic vertical segment">
            {% block content %}
            {% endblock %}
        </section>

        <footer class="ui  vertical basic footer segment" style="background-color: #3E3E3B;">
            <div class="ui container">
                <img class="ui tiny  rounded   image" src="logo_white_padding1.svg">
            </div>
        </footer>
    <!-- SCRIPTS -->
        <script>
            {% block script %} {% endblock %}
        </script>
    </body>
</html>
