#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
"""

A colum/table profiler to detect and extract the potential datetime context of the table
"""